@@ -25,7 +25,7 @@
 #include "KoShapeUserData.h"
 
 class KoTextShapeDataBasePrivate;
-class KoXmlElement;
+#include <KoXmlReaderForward.h>
 class KoShapeLoadingContext;
 class KoShapeSavingContext;
 class KoGenStyle;
@@ -133,7 +133,7 @@ public:
 
 protected:
     /// constructor
-    KoTextShapeDataBase(KoTextShapeDataBasePrivate &);
+    KoTextShapeDataBase(KoTextShapeDataBasePrivate *);
 
     KoTextShapeDataBasePrivate *d_ptr;
 
