@@ -40,8 +40,11 @@ namespace KisLayerUtils
     KRITAIMAGE_EXPORT void sortMergableNodes(KisNodeSP root, QList<KisNodeSP> &inputNodes, QList<KisNodeSP> &outputNodes);
     KRITAIMAGE_EXPORT KisNodeList sortMergableNodes(KisNodeSP root, KisNodeList nodes);
     KRITAIMAGE_EXPORT void filterMergableNodes(KisNodeList &nodes, bool allowMasks = false);
+    KRITAIMAGE_EXPORT KisNodeList sortAndFilterAnyMergableNodesSafe(const KisNodeList &nodes, KisImageSP image);
     KRITAIMAGE_EXPORT bool checkIsChildOf(KisNodeSP node, const KisNodeList &parents);
     KRITAIMAGE_EXPORT void filterUnlockedNodes(KisNodeList &nodes);
+    KRITAIMAGE_EXPORT void refreshHiddenAreaAsync(KisImageSP image, KisNodeSP rootNode, const QRect &preparedArea);
+    KRITAIMAGE_EXPORT QRect recursiveTightNodeVisibleBounds(KisNodeSP rootNode);
 
     /**
      * Returns true if:
@@ -54,6 +57,8 @@ namespace KisLayerUtils
     KRITAIMAGE_EXPORT void forceAllDelayedNodesUpdate(KisNodeSP root);
     KRITAIMAGE_EXPORT bool hasDelayedNodeWithUpdates(KisNodeSP root);
 
+    KRITAIMAGE_EXPORT void forceAllHiddenOriginalsUpdate(KisNodeSP root);
+
     KRITAIMAGE_EXPORT KisNodeList sortAndFilterMergableInternalNodes(KisNodeList nodes, bool allowMasks = false);
 
     KRITAIMAGE_EXPORT void mergeDown(KisImageSP image, KisLayerSP layer, const KisMetaData::MergeStrategy* strategy);
@@ -74,6 +79,8 @@ namespace KisLayerUtils
 
     KRITAIMAGE_EXPORT void changeImageDefaultProjectionColor(KisImageSP image, const KoColor &color);
 
+    KRITAIMAGE_EXPORT bool canChangeImageProfileInvisibly(KisImageSP image);
+
     typedef QMap<int, QSet<KisNodeSP> > FrameJobs;
     void updateFrameJobs(FrameJobs *jobs, KisNodeSP node);
     void updateFrameJobsRecursive(FrameJobs *jobs, KisNodeSP rootNode);
@@ -221,6 +228,16 @@ namespace KisLayerUtils
     KisNodeSP KRITAIMAGE_EXPORT findNodeByUuid(KisNodeSP root, const QUuid &uuid);
 
     KisImageSP KRITAIMAGE_EXPORT findImageByHierarchy(KisNodeSP node);
+
+    template <class T>
+    T* findNodeByType(KisNodeSP root) {
+        return dynamic_cast<T*>(recursiveFindNode(root, [] (KisNodeSP node) {
+            return bool(dynamic_cast<T*>(node.data()));
+        }).data());
+    }
+
+
+    KisNodeSP KRITAIMAGE_EXPORT findRoot(KisNodeSP node);
 }
 
 #endif /* __KIS_LAYER_UTILS_H */
