@@ -1055,9 +1055,10 @@ void KisPaintDeviceTest::testWrappedRandomAccessor()
     int x;
     int y;
 
+    KisRandomAccessorSP dstIt = dev->createRandomAccessorNG();
     x = 3;
     y = 3;
-    KisRandomAccessorSP dstIt = dev->createRandomAccessorNG(x, y);
+    dstIt->moveTo(x, y);
 
     QVERIFY(!memcmp(dstIt->rawData(), c1.data(), pixelSize));
     QCOMPARE(dstIt->numContiguousColumns(x), 17);
@@ -1254,7 +1255,7 @@ void testWrappedLineIteratorReadMoreThanBounds(QString testName)
 
     // test rect doesn't fit the wrap rect in both dimensions
     const QRect &rect(bounds.adjusted(-6,-6,8,8));
-    KisRandomAccessorSP dstIt = dst->createRandomAccessorNG(rect.x(), rect.y());
+    KisRandomAccessorSP dstIt = dst->createRandomAccessorNG();
     IteratorSP it = createIterator<IteratorSP>(dev, rect);
 
     for (int y = rect.y(); y < rect.y() + rect.height(); y++) {
@@ -1358,7 +1359,7 @@ public:
             case 1:
             {
                 int newValue = m_cache.getValue();
-                Q_ASSERT(newValue >= m_oldValue);
+                KIS_ASSERT(newValue >= m_oldValue);
                 Q_UNUSED(newValue);
             }
                 break;
