@@ -234,6 +234,10 @@ Q_SIGNALS:
     /// emitted when the window is migrated among different screens
     void screenChanged();
 
+    /// emitted when the current view has changed
+    void activeViewChanged();
+
+
 public Q_SLOTS:
 
 
@@ -359,6 +363,10 @@ private Q_SLOTS:
 
     void importAnimation();
 
+    void renderAnimation();
+
+    void renderAnimationAgain();
+
     /**
      * Show a dialog with author and document information.
      */
@@ -489,11 +497,6 @@ private:
 
     bool openDocumentInternal(const QUrl &url, KisMainWindow::OpenFlags flags = 0);
 
-
-    /**
-     * Updates the window caption based on the document info and path.
-     */
-    void updateCaption(const QString & caption, bool modified);
     void updateReloadFileAction(KisDocument *doc);
 
     void saveWindowSettings();
