@@ -20,8 +20,8 @@
 #ifndef KGESTUREMAP_H
 #define KGESTUREMAP_H
 
-#include <QtCore/QObject>
-#include <QtCore/QHash>
+#include <QObject>
+#include <QHash>
 #include <QTimer>
 #include <QPolygon>
 
