@@ -91,9 +91,6 @@ KisToolMove::KisToolMove(KoCanvasBase *canvas)
     connect(m_optionsWidget, SIGNAL(sigRequestCommitOffsetChanges()), this, SLOT(commitChanges()), Qt::UniqueConnection);
 
     connect(this, SIGNAL(moveInNewPosition(QPoint)), m_optionsWidget, SLOT(slotSetTranslate(QPoint)), Qt::UniqueConnection);
-
-    connect(qobject_cast<KisCanvas2*>(canvas)->viewManager()->nodeManager(), SIGNAL(sigUiNeedChangeSelectedNodes(KisNodeList)), this, SLOT(slotNodeChanged(KisNodeList)), Qt::UniqueConnection);
-    connect(qobject_cast<KisCanvas2*>(canvas)->viewManager()->selectionManager(), SIGNAL(currentSelectionChanged()), this, SLOT(slotSelectionChanged()), Qt::UniqueConnection);
 }
 
 KisToolMove::~KisToolMove()
@@ -103,43 +100,56 @@ KisToolMove::~KisToolMove()
 
 void KisToolMove::resetCursorStyle()
 {
-    KisTool::resetCursorStyle();
-
     if (!isActive()) return;
-    KisImageSP image = this->image();
-    KisResourcesSnapshotSP resources =
-        new KisResourcesSnapshot(image, currentNode(), canvas()->resourceManager());
-    KisSelectionSP selection = resources->activeSelection();
-    KisNodeList nodes = fetchSelectedNodes(moveToolMode(), &m_lastCursorPos, selection);
 
-    if (nodes.isEmpty()) {
-        canvas()->setCursor(Qt::ForbiddenCursor);
-    }
-}
+    bool canMove = true;
 
-KisNodeList KisToolMove::fetchSelectedNodes(MoveToolMode mode, const QPoint *pixelPoint, KisSelectionSP selection)
-{
-    KisNodeList nodes;
+    if (m_strokeId && m_currentlyUsingSelection) {
+        /// noop; whatever the cursor position, we always show move
+        /// cursor, because we don't use 'layer under cursor' mode
+        /// for moving selections
+    } else if (m_strokeId && !m_currentlyUsingSelection) {
+        /// we cannot pick layer's pixel data while the stroke is running,
+        /// because it may run in lodN mode; therefore, we delegate this
+        /// work to the stroke itself
+        if (m_currentMode != MoveSelectedLayer &&
+            (m_handlesRect.isEmpty() ||
+             !m_handlesRect.translated(currentOffset()).contains(m_lastCursorPos))) {
 
-    KisImageSP image = this->image();
-    if (mode != MoveSelectedLayer && pixelPoint) {
-        const bool wholeGroup = !selection &&  mode == MoveGroup;
-        KisNodeSP node = KisToolUtils::findNode(image->root(), *pixelPoint, wholeGroup);
-        if (node) {
-            nodes = {node};
+            image()->addJob(m_strokeId, new MoveStrokeStrategy::PickLayerData(m_lastCursorPos));
+            return;
+        }
+    } else {
+        KisResourcesSnapshotSP resources =
+            new KisResourcesSnapshot(this->image(), currentNode(), canvas()->resourceManager());
+        KisSelectionSP selection = resources->activeSelection();
+
+        KisPaintLayerSP paintLayer =
+            dynamic_cast<KisPaintLayer*>(this->currentNode().data());
+
+        const bool canUseSelectionMode =
+                paintLayer && selection &&
+                !selection->selectedRect().isEmpty() &&
+                !selection->selectedExactRect().isEmpty();
+
+        if (!canUseSelectionMode) {
+            KisNodeSelectionRecipe nodeSelection =
+                    KisNodeSelectionRecipe(
+                        this->selectedNodes(),
+                        (KisNodeSelectionRecipe::SelectionMode)moveToolMode(),
+                        m_lastCursorPos);
+
+            if (nodeSelection.selectNodesToProcess().isEmpty()) {
+                canMove = false;
+            }
         }
     }
 
-    if (nodes.isEmpty()) {
-        nodes = this->selectedNodes();
-
-        KritaUtils::filterContainer<KisNodeList>(nodes,
-                                                 [](KisNodeSP node) {
-                                                     return node->isEditable();
-                                                 });
+    if (canMove) {
+        KisTool::resetCursorStyle();
+    } else {
+       useCursor(Qt::ForbiddenCursor);
     }
-
-    return nodes;
 }
 
 bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
@@ -151,32 +161,43 @@ bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
         new KisResourcesSnapshot(image, currentNode(), canvas()->resourceManager());
     KisSelectionSP selection = resources->activeSelection();
 
-    KisNodeList nodes = fetchSelectedNodes(mode, pos, selection);
+    KisPaintLayerSP paintLayer =
+        dynamic_cast<KisPaintLayer*>(this->currentNode().data());
 
-    if (nodes.size() == 1) {
-        node = nodes.first();
+    const bool canUseSelectionMode =
+            paintLayer && selection &&
+            !selection->selectedRect().isEmpty() &&
+            !selection->selectedExactRect().isEmpty();
+
+    if (pos) {
+        // finish stroke by clicking outside image bounds
+        if (m_strokeId && !image->bounds().contains(*pos)) {
+            endStroke();
+            return false;
+        }
+
+        // restart stroke when the mode has changed or the user tried to
+        // pick another layer in "layer under cursor" mode.
+        if (m_strokeId &&
+                (m_currentMode != mode ||
+                 m_currentlyUsingSelection != canUseSelectionMode ||
+                 (!m_currentlyUsingSelection &&
+                  mode != MoveSelectedLayer &&
+                  !m_handlesRect.translated(currentOffset()).contains(*pos)))) {
+
+            endStroke();
+        }
     }
 
-    if (nodes.isEmpty()) {
-        return false;
-    }
+    if (m_strokeId) return true;
 
-    /**
-     * If the target node has changed, the stroke should be
-     * restarted. Otherwise just continue processing current node.
-     */
-    if (m_strokeId && !tryEndPreviousStroke(nodes)) {
-        return true;
-    }
+    KisNodeList nodes;
 
     KisStrokeStrategy *strategy;
 
-    KisPaintLayerSP paintLayer = node ?
-        dynamic_cast<KisPaintLayer*>(node.data()) : 0;
-
-    if (paintLayer && selection &&
-        (!selection->selectedRect().isEmpty() &&
-         !selection->selectedExactRect().isEmpty())) {
+    bool isMoveSelection = false;
+    if (canUseSelectionMode) {
+        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(selection, false);
 
         MoveSelectionStrokeStrategy *moveStrategy =
             new MoveSelectionStrokeStrategy(paintLayer,
@@ -187,18 +208,38 @@ bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
         connect(moveStrategy,
                 SIGNAL(sigHandlesRectCalculated(const QRect&)),
                 SLOT(slotHandlesRectCalculated(const QRect&)));
+        connect(moveStrategy,
+                SIGNAL(sigStrokeStartedEmpty()),
+                SLOT(slotStrokeStartedEmpty()));
 
         strategy = moveStrategy;
+        isMoveSelection = true;
+        nodes = {paintLayer};
 
     } else {
+        KisNodeSelectionRecipe nodeSelection =
+            pos ?
+                KisNodeSelectionRecipe(
+                        this->selectedNodes(),
+                        (KisNodeSelectionRecipe::SelectionMode)mode,
+                        *pos) :
+                KisNodeSelectionRecipe(this->selectedNodes());
+
 
         MoveStrokeStrategy *moveStrategy =
-            new MoveStrokeStrategy(nodes, image.data(), image.data());
+            new MoveStrokeStrategy(nodeSelection, image.data(), image.data());
         connect(moveStrategy,
                 SIGNAL(sigHandlesRectCalculated(const QRect&)),
                 SLOT(slotHandlesRectCalculated(const QRect&)));
+        connect(moveStrategy,
+                SIGNAL(sigStrokeStartedEmpty()),
+                SLOT(slotStrokeStartedEmpty()));
+        connect(moveStrategy,
+                SIGNAL(sigLayersPicked(const KisNodeList&)),
+                SLOT(slotStrokePickedLayers(const KisNodeList&)));
 
         strategy = moveStrategy;
+        nodes = nodeSelection.selectedNodes;
     }
 
     // disable outline feedback until the stroke calcualtes
@@ -206,8 +247,14 @@ bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
     m_handlesRect = QRect();
     m_strokeId = image->startStroke(strategy);
     m_currentlyProcessingNodes = nodes;
+    m_currentlyUsingSelection = isMoveSelection;
+    m_currentMode = mode;
     m_accumulatedOffset = QPoint();
 
+    if (!isMoveSelection) {
+        m_asyncUpdateHelper.startUpdateStream(image.data(), m_strokeId);
+    }
+
     KIS_SAFE_ASSERT_RECOVER(m_changesTracker.isEmpty()) {
         m_changesTracker.reset();
     }
@@ -235,13 +282,13 @@ void KisToolMove::notifyGuiAfterMove(bool showFloatingMessage)
     const bool showCoordinates = m_optionsWidget->showCoordinates();
 
     if (showCoordinates && showFloatingMessage) {
-        KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
+        KisCanvas2 *kisCanvas = static_cast<KisCanvas2*>(canvas());
         kisCanvas->viewManager()->
             showFloatingMessage(
                 i18nc("floating message in move tool",
                       "X: %1 px, Y: %2 px",
-                      currentTopLeft.x(),
-                      currentTopLeft.y()),
+                      QLocale().toString(currentTopLeft.x()),
+                      QLocale().toString(currentTopLeft.y())),
                 QIcon(), 1000, KisFloatingMessage::High);
     }
 }
@@ -277,6 +324,37 @@ void KisToolMove::slotHandlesRectCalculated(const QRect &handlesRect)
     notifyGuiAfterMove(false);
 }
 
+void KisToolMove::slotStrokeStartedEmpty()
+{
+    /**
+     * Notify that move-selection stroke ended unexpectedly
+     */
+    if (m_currentlyUsingSelection) {
+        KisCanvas2 *kisCanvas = static_cast<KisCanvas2*>(canvas());
+        kisCanvas->viewManager()->
+            showFloatingMessage(
+                i18nc("floating message in move tool",
+                      "Selected area has no pixels"),
+                QIcon(), 1000, KisFloatingMessage::High);
+    }
+
+    /**
+     * Since the choice of nodes for the operation happens in the
+     * stroke itself, it may happen that there are no nodes at all.
+     * In such a case, we should just cancel already started stroke.
+     */
+    cancelStroke();
+}
+
+void KisToolMove::slotStrokePickedLayers(const KisNodeList &nodes)
+{
+    if (nodes.isEmpty()) {
+        useCursor(Qt::ForbiddenCursor);
+    } else {
+        KisTool::resetCursorStyle();
+    }
+}
+
 void KisToolMove::moveDiscrete(MoveDirection direction, bool big)
 {
     if (mode() == KisTool::PAINT_MODE) return;  // Don't interact with dragging
@@ -328,6 +406,9 @@ void KisToolMove::activate(ToolActivation toolActivation, const QSet<KoShape*> &
     m_actionConnections.addConnection(action("movetool-move-right-more"), SIGNAL(triggered(bool)),
                                       this, SLOT(slotMoveDiscreteRightMore()));
 
+    m_canvasConnections.addUniqueConnection(qobject_cast<KisCanvas2*>(canvas())->viewManager()->nodeManager(), SIGNAL(sigUiNeedChangeSelectedNodes(KisNodeList)), this, SLOT(slotNodeChanged(KisNodeList)));
+    m_canvasConnections.addUniqueConnection(qobject_cast<KisCanvas2*>(canvas())->viewManager()->selectionManager(), SIGNAL(currentSelectionChanged()), this, SLOT(slotSelectionChanged()));
+
     connect(m_showCoordinatesAction, SIGNAL(triggered(bool)), m_optionsWidget, SLOT(setShowCoordinates(bool)), Qt::UniqueConnection);
     connect(m_optionsWidget, SIGNAL(showCoordinatesChanged(bool)), m_showCoordinatesAction, SLOT(setChecked(bool)), Qt::UniqueConnection);
 
@@ -345,7 +426,7 @@ void KisToolMove::paint(QPainter& gc, const KoViewConverter &converter)
 {
     Q_UNUSED(converter);
 
-    if (m_strokeId && !m_handlesRect.isEmpty()) {
+    if (m_strokeId && !m_handlesRect.isEmpty() && !m_currentlyUsingSelection) {
         QPainterPath handles;
         handles.addRect(m_handlesRect.translated(currentOffset()));
 
@@ -357,6 +438,7 @@ void KisToolMove::paint(QPainter& gc, const KoViewConverter &converter)
 void KisToolMove::deactivate()
 {
     m_actionConnections.clear();
+    m_canvasConnections.clear();
 
     disconnect(m_showCoordinatesAction, 0, this, 0);
     disconnect(m_optionsWidget, 0, this, 0);
@@ -436,7 +518,9 @@ void KisToolMove::mouseMoveEvent(KoPointerEvent *event)
     m_lastCursorPos = convertToPixelCoord(event).toPoint();
     KisTool::mouseMoveEvent(event);
 
-    if (moveToolMode() == MoveFirstLayer) {
+    if (moveToolMode() != MoveSelectedLayer ||
+            (m_strokeId && m_currentMode != MoveSelectedLayer)) {
+
         m_updateCursorCompressor.start();
     }
 }
@@ -449,6 +533,13 @@ void KisToolMove::startAction(KoPointerEvent *event, MoveToolMode mode)
 
     if (startStrokeImpl(mode, &pos)) {
         setMode(KisTool::PAINT_MODE);
+
+        if (m_currentlyUsingSelection) {
+            KisImageSP image = currentImage();
+            image->addJob(m_strokeId,
+                          new MoveSelectionStrokeStrategy::ShowSelectionData(false));
+        }
+
     } else {
         event->ignore();
         m_dragPos = QPoint();
@@ -488,6 +579,12 @@ void KisToolMove::endAction(KoPointerEvent *event)
     m_dragPos = QPoint();
     commitChanges();
 
+    if (m_currentlyUsingSelection) {
+        KisImageSP image = currentImage();
+        image->addJob(m_strokeId,
+                      new MoveSelectionStrokeStrategy::ShowSelectionData(true));
+    }
+
     notifyGuiAfterMove();
 
     qobject_cast<KisCanvas2*>(canvas())->updateCanvas();
@@ -495,7 +592,7 @@ void KisToolMove::endAction(KoPointerEvent *event)
 
 void KisToolMove::drag(const QPoint& newPos)
 {
-    KisImageWSP image = currentImage();
+    KisImageSP image = currentImage();
 
     QPoint offset = m_accumulatedOffset + newPos - m_dragStart;
 
@@ -507,11 +604,17 @@ void KisToolMove::endStroke()
 {
     if (!m_strokeId) return;
 
+    if (m_asyncUpdateHelper.isActive()) {
+        m_asyncUpdateHelper.endUpdateStream();
+    }
+
     KisImageSP image = currentImage();
     image->endStroke(m_strokeId);
     m_strokeId.clear();
     m_changesTracker.reset();
     m_currentlyProcessingNodes.clear();
+    m_currentlyUsingSelection = false;
+    m_currentMode = MoveSelectedLayer;
     m_accumulatedOffset = QPoint();
     qobject_cast<KisCanvas2*>(canvas())->updateCanvas();
 }
@@ -573,11 +676,17 @@ void KisToolMove::cancelStroke()
 {
     if (!m_strokeId) return;
 
+    if (m_asyncUpdateHelper.isActive()) {
+        m_asyncUpdateHelper.cancelUpdateStream();
+    }
+
     KisImageSP image = currentImage();
     image->cancelStroke(m_strokeId);
     m_strokeId.clear();
     m_changesTracker.reset();
     m_currentlyProcessingNodes.clear();
+    m_currentlyUsingSelection = false;
+    m_currentMode = MoveSelectedLayer;
     m_accumulatedOffset = QPoint();
     notifyGuiAfterMove();
     qobject_cast<KisCanvas2*>(canvas())->updateCanvas();
