@@ -53,31 +53,51 @@ void KisToolRectangle::resetCursorStyle()
     overrideCursorIfNotEditable();
 }
 
-void KisToolRectangle::finishRect(const QRectF &rect)
+void KisToolRectangle::finishRect(const QRectF &rect, qreal roundCornersX, qreal roundCornersY)
 {
-    if (rect.isNull() || !blockUntilOperationsFinished())
+    if (rect.isNull())
         return;
 
-    if (!currentNode()->inherits("KisShapeLayer")) {
+    const KisToolShape::ShapeAddInfo info =
+        shouldAddShape(currentNode());
+
+    if (!info.shouldAddShape) {
         KisFigurePaintingToolHelper helper(kundo2_i18n("Draw Rectangle"),
                                            image(),
                                            currentNode(),
                                            canvas()->resourceManager(),
                                            strokeStyle(),
-                                           fillStyle());
-        helper.paintRect(rect);
+                                           fillStyle(),
+                                           fillTransform());
+
+        QPainterPath path;
+
+        if (roundCornersX > 0 || roundCornersY > 0) {
+            path.addRoundedRect(rect, roundCornersX, roundCornersY);
+        } else {
+            path.addRect(rect);
+        }
+
+        helper.paintPainterPath(path);
     } else {
-        QRectF r = convertToPt(rect);
-        KoShape* shape = KisShapeToolHelper::createRectangleShape(r);
+        const QRectF r = convertToPt(rect);
+        const qreal docRoundCornersX = convertToPt(roundCornersX);
+        const qreal docRoundCornersY = convertToPt(roundCornersY);
+        KoShape* shape = KisShapeToolHelper::createRectangleShape(r, docRoundCornersX, docRoundCornersY);
 
         KoShapeStrokeSP border;
-        if (strokeStyle() == KisPainter::StrokeStyleBrush) {
-            border = toQShared(new KoShapeStroke(currentStrokeWidth(), currentFgColor().toQColor()));
+        if (strokeStyle() != KisToolShapeUtils::StrokeStyleNone) {
+            const QColor color = strokeStyle() == KisToolShapeUtils::StrokeStyleForeground ?
+                        canvas()->resourceManager()->foregroundColor().toQColor() :
+                        canvas()->resourceManager()->backgroundColor().toQColor();
+
+            border = toQShared(new KoShapeStroke(currentStrokeWidth(), color));
         }
         shape->setStroke(border);
+
+        info.markAsSelectionShapeIfNeeded(shape);
+
         addShape(shape);
     }
-
-    notifyModified();
 }
 
